/*
 * John William Lynch
 * SE: 450
 * Assignment 3
 * Decorator and Chain of Responsibility pattern applied to unit conversion GUI
 * Due : March 12th, 11:59PM
 */
package Assignment3;

import java.awt.Dimension;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;

//Implement Action Listener for picking up input

public class ConverterGUI implements ActionListener {
	
	// Create a var that can hold the content of the user input information:
	//Declare it out of the scope of the function so we can call the getText() method at any location in this class:
	
	private JTextField inputField;
	private JComboBox<String> outputUnit; 
	private  String[] units={"Mile","Yard","Foot"};
    private JFrame frame;
    JTextField outputField = new JTextField(15);
    
    //Create two variables that can return the double and the unit type based on 
    //the user input:
    private double doubleRequest;
    private String unitType;
    
    
    //Constructor to grab the information to create the double:
    public ConverterGUI() {  
        initializeFrame();
  
    }

    // You should add ActionListener for the Convert button
    //Here you have to implement the action listener..
    /*
     * To reiterate, my actionPerformed acts like the Main class as there is a few more layers
     * we must go through to create our GUI. I could not successfully pass data / parse multiple
     * presses of "Convert!" using the constructor of ConverterGUI or the Converter Main
     * Therefore, actionPerformed seemed the best option as it allows the user to constantly
     * update the value and have the value passed through the handlers / decorator:
     */
    @Override
	public void actionPerformed(ActionEvent e) {
		System.out.println("Convert clicked...launching program: " + inputField.getText());
		
		//Grab user inputed number:
		doubleRequest = Double.parseDouble(inputField.getText());
		
		//Grab user selected unit type:
		unitType = String.valueOf(outputUnit.getSelectedItem());
		
		//Pass userInput and unitType to the handlers first, the way the code is to move is from Handler -> Handler -> Handler then to the decorators 
		System.out.println("Retrieved user inputs...");
		
		//Create the Handlers to handle the user inputs:
		UNIT_Handler mileHandler =  new MILE_Handler();
		UNIT_Handler yardHandler = new YARD_Handler();
		UNIT_Handler footHandler = new FOOT_Handler();
		//Set the successors to work properly
		mileHandler.SetSuccessor(yardHandler);
		yardHandler.SetSuccessor(footHandler);
		
		//Start the handling processing by checking if the user entered in MILES:
		//The benefit of this is that I do not account for what type the user puts in here
		//as it will always pass through the chain of command and output the correct value
		/*
		 * Create the three different requests and the successor method will help determine
		 * which one to use:
		 * I attempted to use this only doing a mile Handler, however it would not work
		 * correctly. What occured was that I would successfully convert the value, but then it would
		 * convert it back to the other value afterwards. Therefore I run double through
		 * each handler and what the user picks is what which one will run
		 */
		doubleRequest = mileHandler.HandleRequest(doubleRequest, unitType);
		
		
		//The handlers have now modified the data, now we must retrieve it:
		System.out.println("Successfully converted...passing to decorator:");
		System.out.println(doubleRequest + " " +  unitType);
		
		//doubleRequest and unitType are now found and will need to be passed to the decorators:
		//UserDouble userDouble = new UserDouble();
		//The userDouble is the main double at the start, which will now be decorated using
				//the three required decorators.
		
		/*
		 * To check these run correctly, I removed the conversions and ran it based on 
		 * the input value in the assignment:
		 * 
		Round output to 2nd decimal (e.g., 218.723 to 218.72)
		Write output in exp. notation (e.g., 218.72 to 2.1872e2 )
		Add the unit name to the converted amount (e.g., 2.1872e2 to 2.1872e2 Yards) 
		
		My output was:
		218.723 Mile
		RoundDecorator hit...

		218.72
		ExpDecorator hit...

		2.1872E2
		ValutaDecorator hit...

		2.1872E2 Mile
		
		Now I will add back the conversion rates using the double requests
		 */
		
		UserDouble inputtedValue = new InputDouble(doubleRequest, unitType);
		//Pass to RoundDecorator:
		inputtedValue = new RoundDecorator(inputtedValue);
		System.out.println("First Decoration: " + inputtedValue.currentDouble());
		
		//Pass to ExpDecorator:
		inputtedValue = new ExpDecorator(inputtedValue);
		System.out.println("Second Decoration: " + inputtedValue.currentDouble());
		//Pass to ValutaDecorator:
		inputtedValue = new ValutaDecorator(inputtedValue);
		System.out.println("Third Decoration: " + inputtedValue.currentDouble());
		
		outputField.setText(inputtedValue.currentDouble());
	}
 
 
    private void initializeFrame() {
        frame = new JFrame("Length Converter");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        initUI();
        frame.setLayout(null);
        frame.setPreferredSize(new Dimension(700, 300));
        frame.pack();
        frame.setVisible(true);
    }

   
    private void initUI() {
        inputField = new JTextField(15);
        inputField.setBounds(60, 50, 200, 30);
        JLabel kilometer = new JLabel("Kilometer");
        kilometer.setFont(new Font(Font.MONOSPACED, Font.BOLD, 20));
        kilometer.setBounds (300, 50, 200, 30);
       
        outputField = new JTextField(15);
        outputField.setEditable(false);
        outputField.setLocation(10, 200);
        
        outputField.setBounds(60, 100, 200, 30);
        
        outputUnit = new JComboBox<String>();
        for (String u : units) {
            outputUnit.addItem(u);
        }
        outputUnit.setBounds (300, 100, 60, 30);
        
        //When the user presses the convert button this actionListener fires
        JButton convertButton = new JButton("Convert!");
        convertButton.addActionListener(this);
        
        convertButton.setBounds (60, 150, 200, 30);
        convertButton.setFont(new Font(Font.MONOSPACED, Font.BOLD, 20));
        frame.add(inputField);
        frame.add(kilometer);
        frame.add(outputField);
        frame.add(outputUnit);
        frame.add(convertButton);

    }

    

    
    
    
}
